<div class="modal-header">
	<button type="button" class="close" data-dismiss="modal">&times;</button>
	<h4 class="modal-title">{{ event.tool }} usage details</h4>
</div>
<div class="modal-body">
	User: {{ event.user }}<br>
	Operator: {{ event.operator }}<br>
	Project: {{ event.project.name }}<br>
	Tool: {{ event.tool }}<br>
	Start: {{ event.start }}<br>
	{% if event.end %}
		End: {{ event.end }}<br>
	{% else %}
		This tool is currently in use.<br>
	{% endif %}
	Identifier: {{ event.id }}<br>
</div>