[
	{% for x in events %}
	{
		{% if personal_schedule %}
			"title": "{{ x.title|default:x.tool.name|escapejs }}",
		{% else %}
			"title": "{{ x.title|default:x.user|escapejs }}",
		{% endif %}
		"id": "Reservation {{ x.id }}",
		{# The reservation creator or staff may edit the event: #}
		{% if user.id == x.user.id or user.is_staff %}"editable": true,{% endif %}
		"start": "{{ x.start|date:"c" }}",
		"end": "{{ x.get_visual_end|date:"c" }}",
		"details_url": "{% url 'reservation_details' x.id %}"
	}{% if not forloop.last %},{% elif outages %},{% endif %}
	{% endfor %}
	{% for x in outages %}
	{
		"title": "{{ x.title|escapejs }}",
		"id": "Outage {{ x.id }}",
		{% if user.is_staff %}"editable": true,{% endif %}
		"color": "#ff0000",
		"start": "{{ x.start|date:"c" }}",
		"end": "{{ x.end|date:"c" }}",
		"details_url": "{% url 'outage_details' x.id %}"
	}{% if not forloop.last %},{% endif %}
	{% endfor %}
]