{% extends 'base.html' %}
{% block title %}Contact NanoFab staff{% endblock %}
{% block content %}
	<h1>Contact information</h1>
	{% for category in categories %}
		<h3>{{ category }}</h3>
		<div class="row">
			{% for person in people %}
				{% if person.category == category %}
					<div class="col-md-3">
						<div class="thumbnail">
							{% if person.image %}<img src="{{ person.image.url }}" style="margin-top:7px; height:266px; width:200px">{% endif %}
							<div class="caption">
								<h4 style="margin-top:0; text-align: center">{{ person.name }}</h4>
								{% if person.email %}Email: <a href="mailto:{{ person.email }}">{{ person.email }}</a><br>{% endif %}
								{% if person.office_phone %}Office phone: <a href="tel:{{ person.office_phone }}">{{ person.office_phone }}</a><br>{% endif %}
								{% if person.mobile_phone %}
									Mobile phone:
									{% if person.mobile_phone_is_sms_capable %}
										<ul>
											<li><a href="tel:{{ person.mobile_phone }}">Call {{ person.mobile_phone }}</a></li>
											<li><a href="sms:{{ person.mobile_phone }}">Send a text message</a></li>
										</ul>
									{% else %}
										<a href="tel:{{ person.mobile_phone }}">{{ person.mobile_phone }}</a><br>
									{% endif %}
								{% endif %}
								{% if person.office_location %}Office location: {{ person.office_location }}<br>{% endif %}
							</div>
						</div>
					</div>
				{% endif %}
			{% endfor %}
		</div>
	{% endfor %}
{% endblock %}