{% extends 'base.html' %}
{% block title %}Kiosk locations{% endblock %}
{% block content %}
<h1>Kiosk locations</h1>
{% for location in locations %}
	<a href="{{ location.url }}"><h3>{{ location.name }}</h3></a>
{% endfor %}
{% endblock %}