<script>$('body').removeClass().addClass('logout-warning')</script>
<h1>According to our records, you're not logged in to any access controlled areas.</h1>
<h2>Please visit the NanoFab user office if you believe this is an error.</h2>
<audio src="{% url 'media' 'warning.mp3' %}" autoplay></audio>
<script>revert(15);</script>