<tr id="row_{{ entry_number }}">
	<td style="vertical-align: middle"><span class="glyphicon glyphicon-remove-circle grey pointer" onclick="remove_row({{ entry_number }})"></span></td>
	<td>
		<input type="text" class="form-control" id="user_textbox__{{ entry_number }}" name="chosen_user__{{ entry_number }}" data-row="{{ entry_number }}">
		<input type="button" class="btn btn-default" style="display:none" id="user_button__{{ entry_number }}" onclick="purge_trainee({{ entry_number }})">
	</td>
	<td>
		<input type="text" class="form-control" id="tool_textbox__{{ entry_number }}" name="chosen_tool__{{ entry_number }}" data-row="{{ entry_number }}">
		<input type="button" class="btn btn-default" style="display:none" id="tool_button__{{ entry_number }}" onclick="purge_tool({{ entry_number }})">
	</td>
	<td id="project__{{ entry_number }}" style="vertical-align: middle">

	</td>
	<td><input type="number" class="form-control" name="duration__{{ entry_number }}" id="duration__{{ entry_number }}" min="1" inputmode="numeric" pattern="[0-9]*"></td>
	<td>
		<select class="form-control" name="charge_type__{{ entry_number }}">
			<option value="-1">&nbsp;</option>
			{% for c in charge_types %}
				<option value="{{ c.0 }}">{{ c.1 }}</option>
			{% endfor %}
		</select>
	</td>
	<td style="vertical-align: middle"><input type="checkbox" name="qualify__{{ entry_number }}"></td>
</tr>