{% extends 'base.html' %}
{% block title %}No project{% endblock %}
{% block content %}
	<h1>You are currently not a member of any active projects. You may not perform any billable actions.</h1>
{% endblock %}