<div class="modal-header">
	<button type="button" class="close" data-dismiss="modal">&times;</button>
	<h4 class="modal-title">Scheduled outage details</h4>
</div>
<div class="modal-body">
	Title: {{ outage.title }}<br>
	Created by: {{ outage.creator }}<br>
	{% if outage.tool %}Tool: {{ outage.tool }}<br>{% endif %}
	{% if outage.resource %}Resource: {{ outage.resource }} <span class="light-grey">({{ outage.resource.category }})</span><br>{% endif %}
	{% if outage.category %}Outage category: {{ outage.category }}<br>{% endif %}
	Start: {{ outage.start }}<br>
	End: {{ outage.end }}<br>
	Identifier: {{ outage.id }}<br>
	{% if outage.details %}
		Details:<br>
		{{ outage.details|linebreaksbr }}
	{% endif %}
</div>

{# Allow the user to cancel the outage if they have that priviledge. #}
{% if user.is_staff %}
	<div class="modal-footer">
		<button type="button" class="btn btn-danger" data-dismiss="modal" onclick="$('#dialog').one('hidden.bs.modal', function() { cancel_outage('{% url 'cancel_outage' outage.id %}', 'Outage {{ outage.id }}') })">Cancel this outage</button>
	</div>
{% endif %}